{% load i18n %}

{% if request.breadcrumbs.0 %}
<ul class="breadcrumb">
    {% for breadcrumb in request.breadcrumbs %}
        {% if not forloop.last %}
            <li>
                <a href="{{ breadcrumb.url }}">{{ breadcrumb.name }}</a>
            </li>
        {% else %}
            <li class="active">
                {{ breadcrumb.name }}
            </li>
        {% endif %}
    {% endfor %}
</ul>
{% endif %}